(**************************************************************************)
(*    0_
(*   5|_|1
(*   4|_|2
(*     3
(* 6 -       
(**************************************************************************)

unit hiLedNumberEx; {   ver 2.53  21.08.2022}

interface

uses Win,Share,Windows,Messages,Kol,Debug;

type
  ThiLedNumberEx = class(THIWin)

  private
    FShowLines: Boolean;
    STextAlign: TTextAlign;
    FShadow, FForeground, FBackground: TColor;

    FFonColor, FSegColor, FNoSegColor: TColor;
    FFonHover, FSegHover, FNoSegHover: TColor;
    FPadding, FSpace, FDigWidth, FDigSize, FItalicWidth, FTextWidth, FDotSize, FWidthOutLine: Integer;
    FColorHover, FRound, FItalic, FAutoWidth, FOutLine, FSegmentColon, FTextBits, FAddDot: boolean;
    FText, FDelimiter: String;
    FDrawBuffer: PBitmap;
    ArElRgn: array [0..6] of HRGN;//  ()

    procedure _OnClick( Sender: PObj );
    procedure _OnMouseEnter( Sender: PObj );override;
    procedure _OnMouseLeave( Sender: PObj );override;
    procedure UpdateDrawBuffer;
    procedure CreateSegments;
    procedure ClearSegments;
    
  public
    _prop_Text: String;   
    _event_OnClick: THI_Event;
    _event_OnBeforDraw: THI_Event;
    _event_OnAfterDraw: THI_Event;    
    _data_Text: THI_Event;

    property _prop_TextBits: Boolean       write FTextBits;
    property _prop_Delimiter: string       write FDelimiter;
    property _prop_Alignment: TTextAlign   write STextAlign;
    property _prop_FonColor: TColor        write FFonColor;
    property _prop_SegColor: TColor        write FSegColor;
    property _prop_NoSegColor: TColor      write FNoSegColor;
    property _prop_ShowLines: Boolean      write FShowLines;
    property _prop_OutLine: Boolean        write FOutLine;
    property _prop_WidthOutLine: Integer   write FWidthOutLine;
    property _prop_Round: Boolean          write FRound;
    property _prop_Space: Integer          write FSpace;
    property _prop_Padding: Integer        write FPadding;
    property _prop_Italic: Boolean         write FItalic;
    property _prop_AutoWidth: Boolean      write FAutoWidth;    
    property _prop_DotSize: integer        write FDotSize;
    property _prop_AddDot: Boolean         write FAddDot;    
    property _prop_SegmentColon: Boolean   write FSegmentColon;
    property _prop_ColorHover: Boolean     write FColorHover;
    property _prop_FonHover: TColor        write FFonHover;
    property _prop_SegHover: TColor        write FSegHover;
    property _prop_NoSegHover: TColor      write FNoSegHover;

    destructor Destroy; override;
    procedure Init; override;

    procedure _work_doText(var _Data:TData; Index : word);
    procedure _work_doTextBits(var _Data:TData; Index : word);
    procedure _work_doAlignment(var _Data:TData; Index : word);
    procedure _work_doFonColor(var _Data:TData; Index : word);
    procedure _work_doSegColor(var _Data:TData; Index : word);
    procedure _work_doNoSegColor(var _Data:TData; Index : word);
    procedure _work_doShowLines(var _Data:TData; Index : word);
    procedure _work_doOutLine(var _Data:TData; Index : word);
    procedure _work_doWidthOutLine(var _Data:TData; Index : word);
    procedure _work_doSpace(var _Data:TData; Index : word);
    procedure _work_doPadding(var _Data:TData; Index : word);    
    procedure _work_doRound(var _Data:TData; Index : word);
    procedure _work_doItalic(var _Data:TData; Index : word);
    procedure _work_doAutoWidth(var _Data:TData; Index : word);    
    procedure _work_doDotSize(var _Data:TData; Index : word);
    procedure _work_doAddDot(var _Data:TData; Index : word);
    procedure _work_doSegmentColon(var _Data:TData; Index : word);
    procedure _work_doColorHover(var _Data:TData; Index : word);
    procedure _work_doFonHover(var _Data:TData; Index : word);
    procedure _work_doSegHover(var _Data:TData; Index : word);
    procedure _work_doNoSegHover(var _Data:TData; Index : word);

    procedure _var_Caption(var Data:TData; Index:word);
    procedure _var_Bitmap(var Data:TData; Index:word);
    procedure _var_TextWidth(var Data:TData; Index:word);
    procedure _var_DigitalWidth(var Data:TData; Index:word);
end;

implementation

function StrScan(Str: PChar; Chr: Char): PChar; assembler;
asm
        PUSH    EDI
        PUSH    EAX
        MOV     EDI,Str
        OR      ECX, -1
        XOR     AL,AL
        REPNE   SCASB
        NOT     ECX
        POP     EDI
        XCHG    EAX, EDX
        REPNE   SCASB

        XCHG    EAX, EDI
        POP     EDI

        JE      @@1
        MOV     EAX,1
@@1:    DEC     EAX
end {$IFDEF F_P} [ 'EAX', 'EDX', 'ECX' ] {$ENDIF};

function FParse(var S: string; const Delimiters: char): string;
var
  Pos: Integer;
  P, F: PChar;
begin
  P := PChar(S);
  F := StrScan(P, Delimiters);
  if F <> nil then
    Pos := Integer(F) - Integer(P) + 1
  else
    Pos := Length(S) + 1;    
  Result := S;
  S := CopyEnd(Result, Pos + 1);
  Result := Copy(Result, 1, Pos - 1);
end;

function WndProcLedNumber(Sender: PControl; var Msg: TMsg; var Rslt: Integer ): Boolean;
var   P: TPaintstruct;
      fControl: ThiLedNumberEx;
begin
   fControl := ThiLedNumberEx(Sender.Tag); 
   if msg.Message = WM_PAINT then begin
      BeginPaint(Sender.Handle,P);
      fControl.FDrawbuffer.Draw(Sender.Canvas.Handle, 0, 0);
      EndPaint(Sender.Handle,P);
   end else if msg.message = WM_PRINTCLIENT then begin
      fControl.FDrawbuffer.Draw(hDC(Msg.wParam), 0, 0 );
   end else if msg.message = WM_SIZE then begin
      fControl.fDrawBuffer.width := sender.clientwidth;
      fControl.fDrawBuffer.height := sender.clientheight;
      fControl.CreateSegments;
      fControl.Updatedrawbuffer;
   end else begin
      Result := False;
      exit;
   end;
   Result := True;
   Rslt := 0;
end;

destructor ThiLedNumberEx.Destroy;
begin
   ClearSegments;
   FDrawBuffer.free;
   inherited;
end;

procedure ThiLedNumberEx.Init;
begin
   FText := _prop_Text;
   Control := NewPaintBox(FParent);
   Control.Tag := longint(Self);
   Control.Attachproc(WndProcLedNumber);
   FBackground := FFonColor;
   FForeground := FSegColor;
   FShadow := FNoSegColor;
   FDrawBuffer := NewDIBBitmap(Control.Width, Control.Height, pf32bit);
   Control.OnClick := _OnClick;
   Control.OnMouseEnter := _OnMouseEnter;
   Control.OnMouseLeave := _OnMouseLeave;
inherited;
   //CreateSegments;
   Control.Invalidate;
end;

procedure ThiLedNumberEx.ClearSegments;
var i: integer;
begin
   for i := 0 to 6 do DeleteObject(ArElRgn[i]);
end;

procedure ThiLedNumberEx.CreateSegments;
var p: array [0..5] of TPoint;
    h, hd, rd, zr: integer;
    r: TRect;
    Size: DWORD;
    tmRgn: HRGN;
    wXFORM: XFORM;
    RgnData: PRgnData;
    
    function SetPoint(px, py: integer):TPoint;
    begin
      Result.X := px;
      Result.Y := py;
    end;

begin
    ClearSegments;
    h := Control.Height;
    hd := Control.Height - FPadding * 2;// 
    FDigWidth := 6 * hd div 10;// 
    inc(FDigWidth, ord(odd(FDigWidth)));//  -  
    FDigSize := FDigWidth div 6;// 
    inc(FDigSize, ord(odd(FDigSize)));//  -  
    rd := FDigSize * 2;// 
    zr := max(1, FDigSize div 6);//   
    //   ArElRgn[0]
    if FRound then ArElRgn[0] := CreateRoundRectRgn(0, FPadding, FDigWidth, h, rd, rd)
    else ArElRgn[0] := CreateRectRgn(0, FPadding, FDigWidth, h);
    tmRgn := CreateRectRgn(0, FDigSize + FPadding, FDigWidth, h);
    CombineRgn(ArElRgn[0], ArElRgn[0], tmRgn, RGN_DIFF);// 
    DeleteObject(tmRgn);
    p[0] := SetPoint(0, FPadding);
    p[1] := SetPoint(zr, FPadding);    
    p[2] := SetPoint(FDigSize + zr * 2, FPadding + FDigSize + zr);
    p[3] := SetPoint(0, p[2].y * 2);
    tmRgn := CreatePolygonRgn(p[0], 4, Winding);
    CombineRgn(ArElRgn[0], ArElRgn[0], tmRgn, RGN_DIFF);// 
    DeleteObject(tmRgn);    
    p[0] := SetPoint(FDigWidth, FPadding);
    p[1] := SetPoint(FDigWidth - zr, FPadding);
    p[2] := SetPoint(FDigWidth - FDigSize - zr * 2, p[2].y);
    p[3] := SetPoint(FDigWidth , FPadding +(FDigSize + zr) * 2);
    tmRgn := CreatePolygonRgn(p[0], 4, Winding);
    CombineRgn(ArElRgn[0], ArElRgn[0], tmRgn, RGN_DIFF);//  
    DeleteObject(tmRgn);
    //    ArElRgn[5]
    if FRound then ArElRgn[5] := CreateRoundRectRgn(0, FPadding, FDigWidth, h, rd, rd)
    else ArElRgn[5] := CreateRectRgn(0, FPadding, FDigWidth, h);
    tmRgn := CreateRectRgn(FDigSize, FPadding, FDigWidth, h);
    CombineRgn(ArElRgn[5], ArElRgn[5], tmRgn, RGN_DIFF);
    DeleteObject(tmRgn);
    p[0] := SetPoint(0, FPadding);
    p[1] := SetPoint(0, FPadding + zr);
    p[2] := SetPoint(FDigSize + zr, FPadding + FDigSize + zr * 2);
    p[3] := SetPoint((FDigSize + zr) * 2,  FPadding);
    tmRgn := CreatePolygonRgn(p[0], 4, Winding);
    CombineRgn(ArElRgn[5], ArElRgn[5], tmRgn, RGN_DIFF);// 
    DeleteObject(tmRgn);
    p[0] := SetPoint(0, h div 2 - FDigSize div 2 - zr);//h div 2 - FDigSize div 2 + zr);// 
    p[1] := SetPoint(FDigSize div 2, h div 2 - zr);//
    p[2] := SetPoint(FDigSize,  h div 2 - FDigSize div 2 - zr);// 
    p[3] := SetPoint(FDigSize, h);// 
    p[4] := SetPoint(0,  h);// 
    tmRgn := CreatePolygonRgn(p[0], 5, Winding);
    CombineRgn(ArElRgn[5], ArElRgn[5], tmRgn, RGN_DIFF);//     
    DeleteObject(tmRgn);
    //    Size,  ,    ArElRgn[5]
    //      : ArElRgn[1], ArElRgn[2], ArElRgn[4]
    Size := GetRegionData(ArElRgn[5], 0, nil);
    GetMem(RgnData, SizeOf(RGNDATA) * Size);
    GetRegionData(ArElRgn[5], Size, RgnData);
    //    ArElRgn[4]
    FillChar(wXFORM, SizeOf(wXFORM), #0);
    wXFORM.eM11 := 1;
    wXFORM.eM22 := -1;
    ArElRgn[4] := ExtCreateRegion(@wXFORM, Size, RgnData^);
    OffsetRgn(ArElRgn[4], 0, h);
    //    ArElRgn[2]
    FillChar(wXFORM, SizeOf(wXFORM), #0);
    wXFORM.eM11 := -1;
    wXFORM.eM22 := -1;
    ArElRgn[2] := ExtCreateRegion(@wXFORM, Size, RgnData^);
    OffsetRgn(ArElRgn[2], FDigWidth, h);
    //    ArElRgn[1]
    FillChar(wXFORM, SizeOf(wXFORM), #0);
    wXFORM.eM11 := -1;
    wXFORM.eM22 := 1;
    ArElRgn[1] := ExtCreateRegion(@wXFORM, Size, RgnData^);
    OffsetRgn(ArElRgn[1], FDigWidth, 0);
    FreeMem(RgnData);//     ArElRgn[5]
    //   ArElRgn[3]      ArElRgn[0]
    Size := GetRegionData(ArElRgn[0], 0, nil);
    GetMem(RgnData, SizeOf(RGNDATA) * Size);
    GetRegionData(ArElRgn[0], Size, RgnData);
    FillChar(wXFORM, SizeOf(wXFORM), #0);
    wXFORM.eM11 := 1;
    wXFORM.eM22 := -1;
    ArElRgn[3] := ExtCreateRegion(@wXFORM, Size, RgnData^);
    OffsetRgn(ArElRgn[3], 0, h);
    FreeMem(RgnData);//     ArElRgn[0]
    //   ArElRgn[6]
    p[0] := SetPoint(FDigSize div 2 + zr, h div 2);// 
    p[1] := SetPoint(FDigSize + zr, p[0].y - FDigSize div 2);// 
    p[2] := SetPoint(FDigWidth - p[1].x , p[1].y);// 
    p[3] := SetPoint(FDigWidth - p[0].x, p[0].y);// 
    p[4] := SetPoint(p[2].x, p[0].y + FDigSize div 2);// 
    p[5] := SetPoint(p[1].x , p[4].y);//    
    ArElRgn[6] := CreatePolygonRgn(p[0], 6, Winding);
    if FItalic then//  
     begin
      for rd := 0 to 6 do//  
       begin
        Size := GetRegionData(ArElRgn[rd], 0, nil);
        GetMem(RgnData, SizeOf(RGNDATA) * Size);
        GetRegionData(ArElRgn[rd], Size, RgnData);
        FillChar(wXFORM, SizeOf(wXFORM), #0);
        wXFORM.eM11 := 1;
        wXFORM.eM22 := 1;
        wXFORM.eDx := 0.2;
        wXFORM.eM21 := - wXFORM.eDx;
        DeleteObject(ArElRgn[rd]);
        ArElRgn[rd] := ExtCreateRegion(@wXFORM, Size, RgnData^);
        FreeMem(RgnData);
       end;
      GetRgnBox(ArElRgn[4], r);
      for rd := 0 to 6 do OffsetRgn(ArElRgn[rd], -r.Left, 0);  
      tmRgn := CreateRectRgn(0, 0, 0, 0);
      CombineRgn(tmRgn, ArElRgn[1], ArElRgn[4], RGN_OR);//     
      GetRgnBox(tmRgn, r);//      
      DeleteObject(tmRgn);
      FItalicWidth := r.Right - r.Left - FDigWidth;//   
    end
    else FItalicWidth := 0;
end;

procedure ThiLedNumberEx.UpdateDrawBuffer;
var str: string;
    i, nx, bfD, afD: integer;

  // 
  procedure DrawSymbol(ch: Char; Color: TColor);
  var w, h, iw: integer;
  FRegion, tmpRgn: HRGN;
  br: HBrush;
  begin
   with PCanvas(FDrawBuffer.Canvas){$ifndef F_P}^{$endif} do
    begin
     Pen.Color := Color; 
     if FOutline  
      then Brush.Color := FBackground
      else Brush.Color := Color; 
     case ch of
     '.': begin
           w := nx + bfD;
           h := FDrawBuffer.Height - FPadding;
           if FRound then
             FRegion := CreateEllipticRgn(w, h - FDigSize, w + FDigSize, h)
           else 
	     FRegion := CreateRectRgn(w, h - FDigSize, w + FDigSize, h);
          end;
     ':': begin
           w := nx + bfD;
           h := FDrawBuffer.Height div 2;
           iw := FItalicWidth div 6;
           if FRound then
            begin
             FRegion := CreateEllipticRgn(w, h - FDigSize * 2, w + FDigSize, h - FDigSize);
             tmpRgn := CreateEllipticRgn(w, h + FDigSize * 2, w + FDigSize, h + FDigSize);
             CombineRgn(FRegion, FRegion, tmpRgn, RGN_OR);
             DeleteObject(tmpRgn);
            end
           else
            begin
             FRegion := CreateRectRgn(w + iw, h - FDigSize * 2, w + FDigSize + iw, h - FDigSize);
             tmpRgn := CreateRectRgn(w - iw, h + FDigSize * 2, w + FDigSize - iw, h + FDigSize);
             CombineRgn(FRegion, FRegion, tmpRgn, RGN_OR);
             DeleteObject(tmpRgn);
            end; 
          end;
       else 
	    FRegion := CreateRectRgn(0,0,0,0);  
     end;
    if FOutline and (Color =  FForeground) then
      begin
        br := CreateSolidBrush(FForeground);
        FrameRgn(Handle, FRegion, br, FWidthOutLine, FWidthOutLine); 
        DeleteObject(br);
      end
    else FillRgn(FRegion); 
    end;
    DeleteObject(FRegion);
    inc(nx, bfD + fDigSize + afD + FSpace);//     +   +    +   
  end;

  //  
  procedure DrawSegment(Number: Integer; Color: TColor);
  var bt: integer;
      br: HBrush;
  begin
    with PCanvas(FDrawBuffer.Canvas){$ifndef F_P}^{$endif} do
     begin
      Pen.Color := Color;
      Brush.Color := Color;
      for bt := 0 to 7 do
       if ((Number shr bt) and 1) = 1 then
        begin 
         OffsetRgn(ArElRgn[bt], nx, 0);//  ()   
         if FOutline and (Color =  FForeground) then
          begin
           br := CreateSolidBrush(FForeground);
           FrameRgn(Handle, ArElRgn[bt], br, FWidthOutLine, FWidthOutLine); 
           DeleteObject(br);
          end
         else FillRgn(ArElRgn[bt]); 
         OffsetRgn(ArElRgn[bt], -nx, 0);//    ()
       end;
      if (Color <> fShadow) then
        inc(nx, FDigWidth + FSpace);
      if FAddDot then DrawSymbol('.', Color)//     
       else if Number > 127 then DrawSymbol('.', Color);//   127     
     end;
  end;

begin
TRY
   with Control{$ifndef F_P}^{$endif} do
   begin
    if (height = 0) or (width = 0) then exit;//  
    nx := (FDigWidth - FDigSize) div 2;//       ()
    afD := max(0, Round(nx  / 50 *  FDotSize - nx));//    (FSpace)
    bfD := min(Round(FSpace  / 50 *  FDotSize), FSpace) - FSpace + afD;//     
    nx := FPAdding * 2;//        
    if FTextBits then//     
     begin
      str := FText;
      if pos(FDelimiter[1], str) = 0 then//    
       if str2int(str) > 127//    
       then inc(nx, FDigWidth + FSpace + bfD + FDigSize + afD + FSpace)//   
       else inc(nx, FDigWidth + FSpace)// 
      else//  
       while str <> '' do
        if str2int(fparse(str, FDelimiter[1])) > 127//   
         then inc(nx, FDigWidth + FSpace + afD + FDigSize + bfD + FSpace)//     
         else  inc(nx, FDigWidth + FSpace);//   
     end                                                                   
    else//   
     for i := 0 to Length(FText) - 1 do
      case FText[i + 1] of
       '!': inc(nx, FDigWidth + FSpace + afD + FDigSize + bfD + FSpace);//     
       ':': if FSegmentColon//    
             then inc(nx, FDigWidth + FSpace)
             else inc(nx, afD + FDigSize + bfD + FSpace);
       else inc(nx, FDigWidth + FSpace);
      end;
    FTextWidth := max(FDigWidth + FItalicWidth + FPAdding * 2, nx - FSpace + FItalicWidth);//  
    if FAutoWidth and (FTextWidth <>  width) then Control.Width := FTextWidth;//     
    FDrawBuffer.canvas.Brush.Color := FBackground;
    FDrawBuffer.canvas.Pen.Color := FBackground;
    FDrawBuffer.Canvas.Rectangle(0, 0, width, height);
    _hi_OnEvent(_event_onBeforDraw);
    if FText = '' then exit;
    case STextAlign of 
     taLeft: nx := FPadding;
     taRight: nx := Width - FTextWidth + FPadding;
     taCenter: nx := (Width - FTextWidth) div 2 + FPadding;
    end;
    if FTextBits then
     begin
      if FDelimiter = '' then exit;
      str := FText;
      if pos(FDelimiter[1], str) = 0 then
       begin
        if FShowLines then DrawSegment(127, fShadow);
        DrawSegment(str2int(str), FForeground)
       end
      else
       while str <> '' do
        begin
         if FShowLines then DrawSegment(127, fShadow);
         DrawSegment(str2int(fparse(str, FDelimiter[1])), FForeground);
        end;
     end
    else
     for i := 0 to Length(FText) - 1 do
      begin
       if FShowLines then
        case FText[i + 1] of
         ' ','.': ;
         ':': if FSegmentColon then DrawSegment(127, fShadow);
         else DrawSegment(127, fShadow);
       end;
       case FText[i + 1] of
         ' ': inc(nx, bfD + fDigSize + afD + FSpace);
         '0','O','o': DrawSegment(63, FForeground);
         '1': DrawSegment(6, FForeground);
         '2': DrawSegment(91, FForeground);
         '3': DrawSegment(79, FForeground);
         '4': DrawSegment(102, FForeground);
         '5','S','s': DrawSegment(109, FForeground);
         '6': DrawSegment(125, FForeground);
         '7': DrawSegment(7, FForeground);
         '8': DrawSegment(127, FForeground);
         '9': DrawSegment(111, FForeground);
         'A','a': DrawSegment(119, FForeground);
         'B','b': DrawSegment(124, FForeground);
         'C','c','[','(': DrawSegment(57, FForeground); 
         'D','d': DrawSegment(94, FForeground);
         'E','e': DrawSegment(121, FForeground);
         'F','f': DrawSegment(113, FForeground);
         'G','g': DrawSegment(61, FForeground);
         'H','h': DrawSegment(116, FForeground);
         'I','i': DrawSegment(48, FForeground);
         'J','j': DrawSegment(31, FForeground);
         'K','k': DrawSegment(117, FForeground);
         'L','l': DrawSegment(56, FForeground);
         'M','m': DrawSegment(21, FForeground);
         'N','n': DrawSegment(55, FForeground);
         'P','p': DrawSegment(115, FForeground);
         'Q','q': DrawSegment(103, FForeground);
         'R','r': DrawSegment(51, FForeground);
         'T','t': DrawSegment(120, FForeground);
         'U','u': DrawSegment(62, FForeground);
         'V','v': DrawSegment(46, FForeground);
         'W','w': DrawSegment(42, FForeground);
         'X','x': DrawSegment(118, FForeground);
         'Y','y': DrawSegment(110, FForeground);
         'Z','z': DrawSegment(73, FForeground);
         '\': DrawSegment(96, FForeground);
         ']',')': DrawSegment(15, FForeground);
         '^': DrawSegment(35, FForeground);
         '_': DrawSegment(8, FForeground);
         '!': DrawSegment(130, FForeground);
         '"': DrawSegment(34, FForeground);
         '#': DrawSegment(54, FForeground);
         '$': DrawSegment(105, FForeground);
         '%': DrawSegment(45, FForeground);
         '&': DrawSegment(123, FForeground);
         Char(#39): DrawSegment(32, FForeground);
         '*': DrawSegment(99, FForeground);      
         '+': DrawSegment(112, FForeground);
         ',': DrawSegment(12, FForeground);
         '-': DrawSegment(64, FForeground);       
         '/': DrawSegment(66, FForeground);
         ':': if FSegmentColon then DrawSegment(72, FForeground)
              else DrawSymbol(':', FForeground);
         ';': DrawSegment(76, FForeground);
         '<': DrawSegment(97, FForeground);
         '=': DrawSegment(65, FForeground);
         '>': DrawSegment(67, FForeground);
         '?': DrawSegment(83, FForeground);
         '@': DrawSegment(95, FForeground);
         '.': DrawSymbol('.', FForeground);
        end;
     end;
    _hi_OnEvent(_event_OnAfterDraw);
   end;
FINALLY
    Control.Invalidate;
END;    
end;
   
procedure ThiLedNumberEx._work_doText;
begin
   FText := ReadString(_Data,_data_Text);
   CreateSegments;
   UpdateDrawbuffer;
end;

procedure ThiLedNumberEx._work_doTextBits;
begin
   FTextBits  := ReadBool(_data);
end;

procedure ThiLedNumberEx._work_doAlignment;
begin
   STextAlign := TTextAlign(ToInteger(_Data)); 
   UpdateDrawbuffer;
end;

procedure ThiLedNumberEx._work_doFonColor;
begin
   FFonColor := ToInteger(_Data);
   FBackGround := FFonColor;
   UpdateDrawbuffer;
end;

procedure ThiLedNumberEx._work_doSegColor;
begin
   FSegColor := ToInteger(_Data);
   FForeGround := FSegColor;   
   UpdateDrawbuffer;
end;

procedure ThiLedNumberEx._work_doNoSegColor;
begin
   FNoSegColor := ToInteger(_Data);
   FShadow := FNoSegColor;   
   UpdateDrawbuffer;
end;

procedure ThiLedNumberEx._work_doFonHover;
begin
   FFonHover := ToInteger(_Data);
end;

procedure ThiLedNumberEx._work_doSegHover;
begin
   FSegHover := ToInteger(_Data);
end;

procedure ThiLedNumberEx._work_doNoSegHover;
begin
   FNoSegHover := ToInteger(_Data);
end;

procedure ThiLedNumberEx._work_doColorHover;
begin
   FColorHover := ReadBool(_data);
end;

procedure ThiLedNumberEx._work_doShowLines;
begin
   FShowLines := ReadBool(_data);
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doOutLine;
begin
   FOutLine := ReadBool(_data);
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doWidthOutLine;
begin
   FWidthOutLine := ToInteger(_data);
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doSpace;
begin
   FSpace := ToInteger(_Data);
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doPadding;
begin
   FPadding := ToInteger(_data);
   CreateSegments;
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doRound;
begin
   FRound := ReadBool(_data);
   CreateSegments;
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doItalic;
begin
   FItalic := ReadBool(_data);
   CreateSegments;
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doAutoWidth;
begin
   FAutoWidth := ReadBool(_data);
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doDotSize;
begin
   FDotSize := ToInteger(_data);
   if (FDotSize < 0) or (FDotSize > 100) then FDotSize := 100;
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doAddDot;
begin
   FAddDot := ReadBool(_data);
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._work_doSegmentColon;
begin
   FSegmentColon := ReadBool(_data);
   UpdateDrawbuffer;   
end;

procedure ThiLedNumberEx._OnClick;
begin
   _hi_OnEvent(_event_OnClick);
end;

procedure ThiLedNumberEx._OnMouseEnter;
begin
   if not FColorHover then exit;
   FBackGround := FFonHover;
   FForeGround := FSegHover;
   FShadow := FNoSegHover;
   UpdateDrawbuffer;
end;

procedure ThiLedNumberEx._OnMouseLeave;
begin
   if not FColorHover then exit;
   FBackGround := FFonColor;
   FForeGround := FSegColor;
   FShadow := FNoSegColor;
   UpdateDrawbuffer;
end;

procedure ThiLedNumberEx._var_Caption;
begin
   dtString(Data, FText);
end;

procedure ThiLedNumberEx._var_Bitmap;
begin
  dtBitmap(Data, FDrawBuffer);
end;

procedure ThiLedNumberEx._var_TextWidth;
begin
   dtInteger(Data, FTextWidth);
end;

procedure ThiLedNumberEx._var_DigitalWidth;
begin
   dtInteger(Data, FDigWidth + FItalicWidth);
end;

end.